//
//  MultiFriendEmbededViewController.swift
//  KakaoPartnerSDKSample
//
//  Created by kakao on 2/11/25.
//  Copyright © 2025 apiteam. All rights reserved.
//

import UIKit

import KakaoSDKCommon
import KakaoSDKAuth
import KakaoSDKTalk
import KakaoPartnerSDKTalk
import KakaoPartnerSDKFriendDelegate

class MultiFriendEmbededViewController: BaseEmbededViewController {
    
    private let cntText: UITextView = {
        let textView = UITextView()
        textView.font = .systemFont(ofSize: 15, weight: .bold)
        textView.textColor = .black
        textView.textAlignment = .center
        textView.textContainer.heightTracksTextView = true
        return textView
    }()
    
    private let okBtn: UIButton = {
        let button = UIButton()
        button.setTitle("OK", for: .normal)
        button.setTitleColor(.black, for: .normal)
        button.addTarget(self, action: #selector(getSelectedUsers), for: .touchUpInside)
        
        return button
    }()
    
    private func setOkButtonLayout() {
        naviView.addSubview(okBtn)
        
        okBtn.translatesAutoresizingMaskIntoConstraints = false
        okBtn.topAnchor.constraint(equalTo: naviView.topAnchor).isActive = true
        okBtn.trailingAnchor.constraint(equalTo: naviView.trailingAnchor, constant: -10).isActive = true
        okBtn.bottomAnchor.constraint(equalTo: naviView.bottomAnchor).isActive = true
    }
    
    private func setCntTextLayout() {
        naviView.addSubview(cntText)
        
        cntText.translatesAutoresizingMaskIntoConstraints = false
        cntText.trailingAnchor.constraint(equalTo: okBtn.leadingAnchor).isActive = true
        cntText.topAnchor.constraint(equalTo: naviView.topAnchor).isActive = true
        cntText.bottomAnchor.constraint(equalTo: naviView.bottomAnchor).isActive = true
        cntText.widthAnchor.constraint(equalToConstant: 40).isActive = true
        cntText.heightAnchor.constraint(equalTo: naviView.heightAnchor).isActive = true
    }
    
    override func viewDidLoad() {
        setOkButtonLayout()
        setCntTextLayout()
        getPickerView()
    }
    
    func getPickerView() {
        TalkApi.shared.friendsForPartner(context: PartnerFriendsContext()) { [weak self] friends, error in
            if let error {
                self?.showAlert(value: error)
                return
            }
            
            if let friends, let firstUser = friends.elements?.first {
                self?.getPickerViewWithCustomSection(user: firstUser)
            }
        }
    }
    
    private func getPickerViewWithCustomSection(user: PartnerFriend) {
        guard let appKey = try? KakaoSDK.shared.appKey(),
              let bundleId = Bundle.main.bundleIdentifier,
              let accessToken = TokenManager.manager.getToken()?.accessToken else {
            dismiss(animated: true)
            
            return
        }
                
        let params = PickerFriendRequestParams(showFavorite: false)
        let targetInfo = TargetInfo(appKey: appKey, bundleId: bundleId, accessToken: accessToken)
        
        PickerApi.shared.multiFriendPickerView(
            params: params,
            targetInfo: targetInfo,
            selectedUuids: [user.uuid],
            customSection: CustomSection(title: "Custom Section in Multi", uuids: [user.uuid])
        ) { [weak self] resultView, error in
            guard let self else { return }
            if let error {
                print("##### error occur: \(error)")

                DispatchQueue.main.async {
                    self.dismiss(animated: true)
                }
            }

            if let resultView {
                pickerView = resultView
                setLayout()
                
                pickerView?.setSelectedUuidsCallback(self)
            }
        }
    }
    
    override func viewWillTransition(to size: CGSize, with coordinator: any UIViewControllerTransitionCoordinator) {
        coordinator.animate { [weak self] _ in
            self?.pickerView?.updateSubviewsConstraint()
        }
        super.viewWillTransition(to: size, with: coordinator)
    }
}

extension MultiFriendEmbededViewController: SelectedUuidsCallback {
    func update(uuids: [String]) {
        print("### Multi Picker SelectedUsers : \(uuids)")
        
        DispatchQueue.main.async { [weak self] in
            self?.cntText.text = "\(uuids.count)"
            self?.cntText.centerVertically()
        }
    }
}


extension UITextView {
    /// 텍스트가 차지하는 실제 높이만큼을 계산하고,
    /// 남는 위/아래 여백을 동일하게 만들어 수직 중앙 정렬합니다.
    func centerVertically() {
        // 텍스트가 실제 차지하는 크기 계산
        let fittingSize = CGSize(width: bounds.width, height: .greatestFiniteMagnitude)
        let size = sizeThatFits(fittingSize)
        
        // 전체 높이 대비 텍스트 높이의 차이 / 2가 위쪽 inset
        let topInset = max(0, (bounds.height - size.height) / 2)
        
        // 텍스트 컨테이너 인셋과 contentOffset을 동시에 조절
        textContainerInset.top = topInset
        textContainerInset.bottom = topInset
        // contentOffset을 재조정하지 않으면 위 inset이 먹히지 않을 수 있음
        contentOffset.y = -topInset
    }
    
    /// 레이아웃이 바뀔 때마다 중앙 정렬 로직을 다시 호출
    open override func layoutSubviews() {
        super.layoutSubviews()
        centerVertically()
    }
}
